@@ -1,8 +1,9 @@
-from bson import InvalidBSON, BSON
-
+from bson import InvalidBSON, BSON, codec_options
+import six
 import sys
 import struct
 
+
 class BSONInput(object):
     """
     credit to @mpobrien (https://raw.github.com/mongodb/mongo-hadoop/master/streaming/language_support/python/pymongo_hadoop/input.py)
@@ -20,6 +21,7 @@ class BSONInput(object):
         self.fast_string_prematch=fast_string_prematch
         self.eof = False
         self.decode = decode
+        self.codec = codec_options.CodecOptions(tz_aware=True)
         
     def _read(self):
         try:
@@ -27,13 +29,18 @@ class BSONInput(object):
             size = struct.unpack("<i", size_bits)[0] - 4 # BSON size byte includes itself 
             data = size_bits + self.fh.read(size)
             if len(data) != size + 4:
-                raise struct.error("Unable to cleanly read expected BSON Chunk; EOF, underful buffer or invalid object size.")
-            if data[size + 4 - 1] != "\x00":
+                raise struct.error("Unable to read expected BSON Chunk; " +
+                                   "EOF, underful buffer or invalid object size.")
+            if six.PY3:
+                eoo = 0x00
+            else:  # six.PY2
+                eoo = "\x00"
+            if data[size + 4 - 1] != eoo:
                 raise InvalidBSON("Bad EOO in BSON Data")
-            if self.fast_string_prematch in data:
+            if self.fast_string_prematch.encode("utf-8") in data:
                 if self.decode:
                     try:
-                        return BSON(data).decode(tz_aware=True)
+                        return BSON(data).decode(self.codec)
                     except TypeError:
                         return BSON(data).decode()
                 else:
@@ -64,14 +71,17 @@ class KeyValueBSONInput(BSONInput):
     def read(self):
         try:
             doc = self._read()
-        except StopIteration, e:
-            #print >> sys.stderr, "Key/Value Input iteration failed/stopped: %s" % e
+        except StopIteration as e:
+            # print >> sys.stderr, "Key/Value Input iteration failed/stopped: %s" % e
             return None
-        return  doc
+        return doc
 
     def reads(self):
         it = self._reads()
-        n = it.next
+        if six.PY3:
+            n = it.__next__
+        else:  # six.PY2
+            n = it.next
         while 1:
             doc = n()
             yield doc
